import imdb
import json
from pathlib import Path


def _get_actor_name_from_user():
    name = input('Please enter the name of the actor/actress you want you know about: ').split(' ')
    return "{}, {}".format(name[1], name[0])


def _get_movie_title_and_year(film):
    title = film.data.get('title')
    title = title if '()' not in title else title[:-3]
    year = film.data.get('year')
    year = year if year is not None else 3000  # the sorted() method will crash if passed None so return 3000
    return {'title': title, 'year': year}


def _get_json_object_from_actor(actor, reverse):
    actor_or_actress = [s for s in actor['filmography'][0] if 'act' in s][0]
    films = [_get_movie_title_and_year(film) for film
             in actor['filmography'][0].get(actor_or_actress)
             if film.data.get('kind') == 'movie']
    actual_films = []
    for film in sorted(films, key=lambda film: film['year'], reverse=reverse):
        year_value = film.get('year')
        actual_films.append({'name': film.get('title'),
                             'year': year_value if year_value != 3000 else 'unreleased'})
    dict_object = {
        'actor_name': actor.data['name'],
        'films': actual_films
    }
    json_object = json.dumps(dict_object, indent=4)
    return json_object


def _interact_with_actor(actor):
    print('Retrieving movie information for {}'.format(actor['name']))
    actor_or_actress = [s for s in actor['filmography'][0] if 'act' in s][0]
    filmography = actor['filmography'][0].get(actor_or_actress)  # [0] index needed?

    if filmography is None:
        print("{} has not been in any movies.".format(actor['name']))
        return

    print("\n{} has been in {} movies.".format(actor['name'], len(filmography)))

    while True:
        print("Please select an option: ")
        print("\tlist [-r]\t\twill print movies chronologically (providing the optional -r flag will reverse order)")
        print("\tjson [-r] <save_directory>\t\twill save to a json file (providing the optional -r flag will reverse "
              "order; defaults to current directory if <save_directory> is not provided)")
        print("\tquit\t\twill stop interacting with {} and search for another actor".format(actor['name']))

        answer = input("Your choice: ").split(' ')  # too primitive??

        if answer[0] == 'list':
            reverse = False

            if len(answer) > 1:
                reverse = True if answer[1] == '-r' else False

            film_list = [_get_movie_title_and_year(film) for film in filmography if film.data.get('kind') == 'movie']

            # We sort on the second item in the tuple which is the year.
            for film in sorted(film_list, key=lambda film: film['year'], reverse=reverse):
                year_value = film.get('year')
                print("{} ({})".format(film.get('title'), year_value if year_value != 3000 else 'unreleased'))

        elif answer[0] == 'json':
            reverse = False
            file_location = Path().absolute()

            if len(answer) > 1:
                if answer[1] == '-r':
                    reverse = True if answer[1] == '-r' else False
                    if len(answer) > 2:
                        file_location = answer[2]
                else:
                    file_location = answer[1]

            path = Path(file_location / "{}.json".format(actor['name']))
            with open(path, mode='wt') as f:
                f.write(_get_json_object_from_actor(actor, reverse))

            print('{}.json file successfully saved!'.format(actor['name']))

        elif answer[0] == 'quit':
            break

        else:
            print('Not valid input. Please try again.')


        input('PRESS ENTER TO CONTINUE')


def _get_int_from_user_input():
    while True:
        answer = input('Please select the number of the actor you would like to see: ')
        try:
            integer = int(answer)
            return integer
        except ValueError:
            print('{} is not an integer.'.format(answer))
        except Exception as ex:
            print('Error: {}'.format(str(ex)))


def imdb_actor_search():
    global ia
    ia = imdb.IMDb()
    print('Welcome to the IMDb python program!!')
    while True:
        actor_name_query = _get_actor_name_from_user()
        print("Please wait a moment while we retrieve information about {} from IMDb".format(actor_name_query))
        actor_results = ia.search_person(actor_name_query)
        # List comes back with many actors with similar names so perform a direct name search on results
        # to see if we get a single exact match.
        actor = [x for x in actor_results if x.data['name'] == actor_name_query]

        if len(actor) == 1:  # We have found exact match.
            actor_info = ia.get_person(actor[0].personID)
            # _interact_with_actor(actor_info)
        else:
            print("There were {0} actors found with the name '{1}'".format(len(actor_results), actor_name_query))
            for index, actor in enumerate(actor_results):
                print("{0} - {1}".format(index, actor['name']))
            actor_index = _get_int_from_user_input()
            actor_info = ia.get_person(actor_results[actor_index].personID)

        _interact_with_actor(actor_info)


def main():
    imdb_actor_search()


if __name__ == '__main__':
    main()
